@@ -39,6 +39,7 @@
 #include "gviewv4l2core.h"
 #include "video_capture.h"
 #include "gviewencoder.h"
+#include "gviewrender.h"
 #include "gui.h"
 #include "gui_gtk3.h"
 #include "core_io.h"
@@ -114,11 +115,9 @@ void camera_button_menu_changed (GtkWidget *item, void *data)
  */
 void control_defaults_clicked (GtkWidget *item, void *data)
 {
-    v4l2_dev_t *device = (v4l2_dev_t *) data;
+    v4l2core_set_control_defaults();
 
-    v4l2core_set_control_defaults(device);
-
-    gui_gtk3_update_controls_state(device);
+    gui_gtk3_update_controls_state();
 }
 
 /*
@@ -166,8 +165,6 @@ static void profile_update_extension (GtkComboBox *chooser, GtkWidget *file_dial
  */
 void controls_profile_clicked (GtkWidget *item, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	GtkWidget *FileDialog;
 
 	int save_or_load = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (item), "profile_dialog"));
@@ -243,12 +240,12 @@ void controls_profile_clicked (GtkWidget *item, void *data)
 
 		if(save_or_load > 0)
 		{
-			v4l2core_save_control_profile(device, filename);
+			v4l2core_save_control_profile(filename);
 		}
 		else
 		{
-			v4l2core_load_control_profile(device, filename);
-			gui_gtk3_update_controls_state(device);
+			v4l2core_load_control_profile(filename);
+			gui_gtk3_update_controls_state();
 		}
 
 		char *basename = get_file_basename(filename);
@@ -510,8 +507,6 @@ static void video_update_extension (GtkComboBox *chooser, GtkWidget *file_dialog
  */
 void photo_file_clicked (GtkWidget *item, void *data)
 {
-	//v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	GtkWidget *FileDialog;
 
 	GtkWidget *main_window = get_main_window_gtk3();
@@ -612,8 +607,6 @@ void photo_file_clicked (GtkWidget *item, void *data)
  */
 void video_file_clicked (GtkWidget *item, void *data)
 {
-	//v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	GtkWidget *FileDialog;
 
 	GtkWidget *main_window = get_main_window_gtk3();
@@ -736,8 +729,6 @@ void capture_image_clicked (GtkButton *button, void *data)
  */
 void capture_video_clicked(GtkToggleButton *button, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	int active = gtk_toggle_button_get_active (button);
 
 	if(debug_level > 0)
@@ -745,7 +736,7 @@ void capture_video_clicked(GtkToggleButton *button, void *data)
 
 	if(active)
 	{
-		start_encoder_thread(device);
+		start_encoder_thread();
 		gtk_button_set_label(GTK_BUTTON(button), _("Stop Video (V)"));
 
 	}
@@ -772,16 +763,14 @@ void capture_video_clicked(GtkToggleButton *button, void *data)
  */
 void pan_tilt_step_changed (GtkSpinButton *spin, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (spin), "control_info"));
 
 	int val = gtk_spin_button_get_value_as_int (spin);
 
 	if(id == V4L2_CID_PAN_RELATIVE)
-		device->pan_step = val;
+		v4l2core_set_pan_step(val);
 	if(id == V4L2_CID_TILT_RELATIVE)
-		device->tilt_step = val;
+		v4l2core_set_tilt_step(val);
 }
 
 /*
@@ -797,18 +786,16 @@ void pan_tilt_step_changed (GtkSpinButton *spin, void *data)
  */
 void button_PanTilt1_clicked (GtkButton * Button, void *data)
 {
-    v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
 
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 	if(id == V4L2_CID_PAN_RELATIVE)
-		control->value =  device->pan_step;
+		control->value = v4l2core_get_pan_step();
 	else
-		control->value =  device->tilt_step;
+		control->value = v4l2core_get_tilt_step();
 
-    if(v4l2core_set_control_value_by_id(device, id))
+    if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting pan/tilt\n");
 }
 
@@ -825,18 +812,16 @@ void button_PanTilt1_clicked (GtkButton * Button, void *data)
  */
 void button_PanTilt2_clicked (GtkButton * Button, void *data)
 {
-    v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
 
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
     if(id == V4L2_CID_PAN_RELATIVE)
-		control->value =  - device->pan_step;
+		control->value =  - v4l2core_get_pan_step();
 	else
-		control->value =  - device->tilt_step;
+		control->value =  - v4l2core_get_tilt_step();
 
-    if(v4l2core_set_control_value_by_id(device, id))
+    if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting pan/tilt\n");
 }
 
@@ -853,18 +838,16 @@ void button_PanTilt2_clicked (GtkButton * Button, void *data)
  */
 void button_clicked (GtkButton * Button, void *data)
 {
-    v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
 
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 	control->value = 1;
 
-    if(v4l2core_set_control_value_by_id(device, id))
+    if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting button value\n");
 
-	gui_gtk3_update_controls_state(device);
+	gui_gtk3_update_controls_state();
 }
 
 #ifdef V4L2_CTRL_TYPE_STRING
@@ -881,18 +864,16 @@ void button_clicked (GtkButton * Button, void *data)
  */
 void string_button_clicked(GtkButton * Button, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
 	GtkWidget *entry = (GtkWidget *) g_object_get_data (G_OBJECT (Button), "control_entry");
 
-	v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+	v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 	assert(control->string != NULL);
 
 	strncpy(control->string, gtk_entry_get_text(GTK_ENTRY(entry)), control->control.maximum);
 
-	if(v4l2core_set_control_value_by_id(device, id))
+	if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting string value\n");
 }
 #endif
@@ -911,12 +892,10 @@ void string_button_clicked(GtkButton * Button, void *data)
  */
 void int64_button_clicked(GtkButton * Button, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
 	GtkWidget *entry = (GtkWidget *) g_object_get_data (G_OBJECT (Button), "control_entry");
 
-	v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+	v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 	char* text_input = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
 	text_input = g_strstrip(text_input);
@@ -932,7 +911,7 @@ void int64_button_clicked(GtkButton * Button, void *data)
 	}
 	g_free(text_input);
 
-	if(v4l2core_set_control_value_by_id(device, id))
+	if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting string value\n");
 
 }
@@ -952,19 +931,17 @@ void int64_button_clicked(GtkButton * Button, void *data)
  */
 void bitmask_button_clicked(GtkButton * Button, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (Button), "control_info"));
 	GtkWidget *entry = (GtkWidget *) g_object_get_data (G_OBJECT (Button), "control_entry");
 
-	v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+	v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 	char* text_input = g_strdup(gtk_entry_get_text(GTK_ENTRY(entry)));
 	text_input = g_strcanon(text_input,"0123456789ABCDEFabcdef", '\0');
 	control->value = (int32_t) g_ascii_strtoll(text_input, NULL, 16);
 	g_free(text_input);
 
-	if(v4l2core_set_control_value_by_id(device, id))
+	if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting string value\n");
 }
 #endif
@@ -982,16 +959,14 @@ void bitmask_button_clicked(GtkButton * Button, void *data)
  */
 void slider_changed (GtkRange * range, void *data)
 {
-    v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (range), "control_info"));
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
     int val = (int) gtk_range_get_value (range);
 
     control->value = val;
 
-    if(v4l2core_set_control_value_by_id(device, id))
+    if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting slider value\n");
 
    /*
@@ -1021,15 +996,13 @@ void slider_changed (GtkRange * range, void *data)
  */
 void spin_changed (GtkSpinButton * spin, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (spin), "control_info"));
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 	int val = gtk_spin_button_get_value_as_int (spin);
     control->value = val;
 
-     if(v4l2core_set_control_value_by_id(device, id))
+     if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting spin value\n");
 
 	/*
@@ -1060,18 +1033,16 @@ void spin_changed (GtkSpinButton * spin, void *data)
  */
 void combo_changed (GtkComboBox * combo, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (combo), "control_info"));
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
     int index = gtk_combo_box_get_active (combo);
     control->value = control->menu[index].index;
 
-	if(v4l2core_set_control_value_by_id(device, id))
+	if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting menu value\n");
 
-	gui_gtk3_update_controls_state(device);
+	gui_gtk3_update_controls_state();
 }
 
 /*
@@ -1087,12 +1058,10 @@ void combo_changed (GtkComboBox * combo, void *data)
  */
 void bayer_pix_ord_changed (GtkComboBox * combo, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	//int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (combo), "control_info"));
 
 	int index = gtk_combo_box_get_active (combo);
-	device->bayer_pix_order = index;
+	v4l2core_set_bayer_pix_order(index);
 }
 
 /*
@@ -1108,34 +1077,32 @@ void bayer_pix_ord_changed (GtkComboBox * combo, void *data)
  */
 void check_changed (GtkToggleButton *toggle, void *data)
 {
-    v4l2_dev_t *device = (v4l2_dev_t *) data;
-
     int id = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (toggle), "control_info"));
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
     int val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
 
     control->value = val;
 
-	if(v4l2core_set_control_value_by_id(device, id))
+	if(v4l2core_set_control_value_by_id(id))
 		fprintf(stderr, "GUVCVIEW: error setting menu value\n");
 
     if(id == V4L2_CID_DISABLE_PROCESSING_LOGITECH)
     {
         if (control->value > 0)
-			device->isbayer = 1;
+			v4l2core_set_isbayer(1);
         else
-			device->isbayer = 0;
+			v4l2core_set_isbayer(0);
 
         /*
          * must restart stream and requeue
          * the buffers for changes to take effect
          * (updating fps provides all that is needed)
          */
-        v4l2core_request_framerate_update (device);
+        v4l2core_request_framerate_update ();
     }
 
-    gui_gtk3_update_controls_state(device);
+    gui_gtk3_update_controls_state();
 }
 
 /*
@@ -1151,12 +1118,10 @@ void check_changed (GtkToggleButton *toggle, void *data)
  */
 void devices_changed (GtkComboBox *wgtDevices, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	GError *error = NULL;
 
 	int index = gtk_combo_box_get_active(wgtDevices);
-	if(index == device->this_device)
+	if(index == v4l2core_get_this_device_index())
 		return;
 
 	v4l2_device_list *device_list = v4l2core_get_device_list();
@@ -1217,7 +1182,7 @@ void devices_changed (GtkComboBox *wgtDevices, void *data)
 			break;
 	}
 	/*reset to current device*/
-	gtk_combo_box_set_active(GTK_COMBO_BOX(wgtDevices), device->this_device);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(wgtDevices), v4l2core_get_this_device_index());
 
 	gtk_widget_destroy (restartdialog);
 	g_free(command);
@@ -1236,25 +1201,26 @@ void devices_changed (GtkComboBox *wgtDevices, void *data)
  */
 void frame_rate_changed (GtkComboBox *wgtFrameRate, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
-	int format_index = v4l2core_get_frame_format_index(device, device->requested_fmt);
+	int format_index = v4l2core_get_frame_format_index(v4l2core_get_requested_frame_format());
 
 	int resolu_index = v4l2core_get_format_resolution_index(
-		device,
 		format_index,
-		device->format.fmt.pix.width,
-		device->format.fmt.pix.height);
+		v4l2core_get_frame_width(),
+		v4l2core_get_frame_height());
 
 	int index = gtk_combo_box_get_active (wgtFrameRate);
 
-	device->fps_denom = device->list_stream_formats[format_index].list_stream_cap[resolu_index].framerate_denom[index];
-	device->fps_num = device->list_stream_formats[format_index].list_stream_cap[resolu_index].framerate_num[index];
+	v4l2_stream_formats_t *list_stream_formats = v4l2core_get_formats_list();
+	
+	int fps_denom = list_stream_formats[format_index].list_stream_cap[resolu_index].framerate_denom[index];
+	int fps_num = list_stream_formats[format_index].list_stream_cap[resolu_index].framerate_num[index];
+	
+	v4l2core_define_fps(fps_num, fps_denom);
 
-	int fps[2] = {device->fps_num, device->fps_denom};
+	int fps[2] = {fps_num, fps_denom};
 	gui_set_fps(fps);
 
-	v4l2core_request_framerate_update (device);
+	v4l2core_request_framerate_update ();
 }
 
 /*
@@ -1270,9 +1236,7 @@ void frame_rate_changed (GtkComboBox *wgtFrameRate, void *data)
  */
 void resolution_changed (GtkComboBox *wgtResolution, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
-	int format_index = v4l2core_get_frame_format_index(device, device->requested_fmt);
+	int format_index = v4l2core_get_frame_format_index(v4l2core_get_requested_frame_format());
 
 	int cmb_index = gtk_combo_box_get_active(wgtResolution);
 
@@ -1281,31 +1245,33 @@ void resolution_changed (GtkComboBox *wgtResolution, void *data)
 	char temp_str[20];
 
 	/*disable fps combobox signals*/
-	g_signal_handlers_block_by_func(GTK_COMBO_BOX_TEXT(wgtFrameRate), G_CALLBACK (frame_rate_changed), device);
+	g_signal_handlers_block_by_func(GTK_COMBO_BOX_TEXT(wgtFrameRate), G_CALLBACK (frame_rate_changed), NULL);
 	/* clear out the old fps list... */
 	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model (GTK_COMBO_BOX(wgtFrameRate)));
 	gtk_list_store_clear(store);
 
-	int width = device->list_stream_formats[format_index].list_stream_cap[cmb_index].width;
-	int height = device->list_stream_formats[format_index].list_stream_cap[cmb_index].height;
+	v4l2_stream_formats_t *list_stream_formats = v4l2core_get_formats_list();
+	
+	int width = list_stream_formats[format_index].list_stream_cap[cmb_index].width;
+	int height = list_stream_formats[format_index].list_stream_cap[cmb_index].height;
 
 	/*check if frame rate is available at the new resolution*/
 	int i=0;
 	int deffps=0;
 
-	for ( i = 0 ; i < device->list_stream_formats[format_index].list_stream_cap[cmb_index].numb_frates ; i++)
+	for ( i = 0 ; i < list_stream_formats[format_index].list_stream_cap[cmb_index].numb_frates ; i++)
 	{
 		g_snprintf(
 			temp_str,
 			18,
 			"%i/%i fps",
-			device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom[i],
-			device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num[i]);
+			list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom[i],
+			list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num[i]);
 
 		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(wgtFrameRate), temp_str);
 
-		if (( device->fps_num == device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num[i]) &&
-			( device->fps_denom == device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom[i]))
+		if (( v4l2core_get_fps_num() == list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num[i]) &&
+			( v4l2core_get_fps_denom() == list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom[i]))
 				deffps=i;
 	}
 
@@ -1313,16 +1279,16 @@ void resolution_changed (GtkComboBox *wgtResolution, void *data)
 	gtk_combo_box_set_active(GTK_COMBO_BOX(wgtFrameRate), deffps);
 
 	/*enable fps combobox signals*/
-	g_signal_handlers_unblock_by_func(GTK_COMBO_BOX_TEXT(wgtFrameRate), G_CALLBACK (frame_rate_changed), device);
+	g_signal_handlers_unblock_by_func(GTK_COMBO_BOX_TEXT(wgtFrameRate), G_CALLBACK (frame_rate_changed), NULL);
 
-	if (device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num)
-		device->fps_num = device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num[deffps];
+	if (list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num)
+		v4l2core_define_fps(list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_num[deffps], -1);
 
-	if (device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom)
-		device->fps_denom = device->list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom[deffps];
+	if (list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom)
+		v4l2core_define_fps(-1, list_stream_formats[format_index].list_stream_cap[cmb_index].framerate_denom[deffps]);
 
 	/*change resolution (try new format and reset render)*/
-	v4l2core_prepare_new_resolution(device, width, height);
+	v4l2core_prepare_new_resolution(width, height);
 
 	request_format_update();
 
@@ -1346,8 +1312,6 @@ void resolution_changed (GtkComboBox *wgtResolution, void *data)
  */
 void format_changed(GtkComboBox *wgtInpType, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	char temp_str[20];
 	int index = gtk_combo_box_get_active(wgtInpType);
 
@@ -1358,43 +1322,45 @@ void format_changed(GtkComboBox *wgtInpType, void *data)
 	int defres = 0;
 
 	/*disable resolution combobox signals*/
-	g_signal_handlers_block_by_func(GTK_COMBO_BOX_TEXT(wgtResolution), G_CALLBACK (resolution_changed), device);
+	g_signal_handlers_block_by_func(GTK_COMBO_BOX_TEXT(wgtResolution), G_CALLBACK (resolution_changed), NULL);
 
 	/* clear out the old resolution list... */
 	GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model (GTK_COMBO_BOX(wgtResolution)));
 	gtk_list_store_clear(store);
 
-	int format = device->list_stream_formats[index].format;
+	v4l2_stream_formats_t *list_stream_formats = v4l2core_get_formats_list();
+		
+	int format = list_stream_formats[index].format;
 
 	/*update config*/
 	config_t *my_config = config_get();
-	strncpy(my_config->format, device->list_stream_formats[index].fourcc, 4);
+	strncpy(my_config->format, list_stream_formats[index].fourcc, 4);
 
 	/*redraw resolution combo for new format*/
-	for(i = 0 ; i < device->list_stream_formats[index].numb_res ; i++)
+	for(i = 0 ; i < list_stream_formats[index].numb_res ; i++)
 	{
-		if (device->list_stream_formats[index].list_stream_cap[i].width > 0)
+		if (list_stream_formats[index].list_stream_cap[i].width > 0)
 		{
 			g_snprintf(
 				temp_str,
 				18,
 				"%ix%i",
-				device->list_stream_formats[index].list_stream_cap[i].width,
-				device->list_stream_formats[index].list_stream_cap[i].height);
+				list_stream_formats[index].list_stream_cap[i].width,
+				list_stream_formats[index].list_stream_cap[i].height);
 
 			gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(wgtResolution), temp_str);
 
-			if ((device->format.fmt.pix.width == device->list_stream_formats[index].list_stream_cap[i].width) &&
-				(device->format.fmt.pix.height == device->list_stream_formats[index].list_stream_cap[i].height))
+			if ((v4l2core_get_frame_width() == list_stream_formats[index].list_stream_cap[i].width) &&
+				(v4l2core_get_frame_height() == list_stream_formats[index].list_stream_cap[i].height))
 					defres=i;//set selected resolution index
 		}
 	}
 
 	/*enable resolution combobox signals*/
-	g_signal_handlers_unblock_by_func(GTK_COMBO_BOX_TEXT(wgtResolution), G_CALLBACK (resolution_changed), device);
+	g_signal_handlers_unblock_by_func(GTK_COMBO_BOX_TEXT(wgtResolution), G_CALLBACK (resolution_changed), NULL);
 
 	/*prepare new format*/
-	v4l2core_prepare_new_format(device, format);
+	v4l2core_prepare_new_format(format);
 	/*change resolution*/
 	gtk_combo_box_set_active(GTK_COMBO_BOX(wgtResolution), defres);
 }
@@ -1443,6 +1409,35 @@ void audio_fx_filter_changed(GtkToggleButton *toggle, void *data)
 	set_audio_fx_mask(mask);
 }
 
+/*
+ * render osd changed event
+ * args:
+ *    toggle - widget that generated the event
+ *    data - pointer to user data
+ *
+ * asserts:
+ *    none
+ *
+ * returns: none
+ */
+void render_osd_changed(GtkToggleButton *toggle, void *data)
+{
+	int osd = GPOINTER_TO_INT(g_object_get_data (G_OBJECT (toggle), "osd_info"));
+
+	uint32_t mask = gtk_toggle_button_get_active (toggle) ?
+			render_get_osd_mask() | osd :
+			render_get_osd_mask() & ~osd;
+
+	render_set_osd_mask(mask);
+
+	/* update config */
+	config_t *my_config = config_get();
+	my_config->osd_mask = render_get_osd_mask();
+	/*make sure to disable VU meter OSD in config - it's set by audio capture*/
+	my_config->osd_mask &= ~REND_OSD_VUMETER_MONO;
+	my_config->osd_mask &= ~REND_OSD_VUMETER_STEREO;
+}
+
 /*
  * software autofocus checkbox changed event
  * args:
@@ -1456,8 +1451,6 @@ void audio_fx_filter_changed(GtkToggleButton *toggle, void *data)
  */
 void autofocus_changed (GtkToggleButton * toggle, void *data)
 {
-    //v4l2_dev_t *device = (v4l2_dev_t *) data;
-
 	int val = gtk_toggle_button_get_active (toggle) ? 1 : 0;
 
 	GtkWidget *wgtFocus_slider = (GtkWidget *) g_object_get_data (G_OBJECT (toggle), "control_entry");
@@ -1509,15 +1502,16 @@ void audio_device_changed(GtkComboBox *combo, void *data)
 	audio_context_t *audio_ctx = get_audio_context();
 
 	if(index < 0)
-		audio_ctx->device = 0;
+		index = 0;
 	else if (index >= audio_ctx->num_input_dev)
-		audio_ctx->device = audio_ctx->num_input_dev - 1;
-	else
-		audio_ctx->device = index;
-
+		index = audio_ctx->num_input_dev - 1;
+	
 	/*update config*/
 	config_t *my_config = config_get();
 	my_config->audio_device = index;
+	
+	/*set the audio device defaults*/
+	audio_set_device(audio_ctx, my_config->audio_device);
 
 	if(debug_level > 0)
 		printf("GUVCVIEW: audio device changed to %i\n", audio_ctx->device);
@@ -1535,6 +1529,8 @@ void audio_device_changed(GtkComboBox *combo, void *data)
 
 	if(index == 0)
 		audio_ctx->samprate = audio_ctx->list_devices[audio_ctx->device].samprate;
+		
+	gtk_range_set_value(GTK_RANGE(my_audio_widgets->latency), audio_ctx->latency);
 }
 
 /*
@@ -1664,7 +1660,7 @@ void audio_api_changed(GtkComboBox *combo, void *data)
 	int api = gtk_combo_box_get_active(combo);
 
 	/*update the audio context for the new api*/
-	audio_context_t *audio_ctx = create_audio_context(api);
+	audio_context_t *audio_ctx = create_audio_context(api, -1);
 	if(!audio_ctx)
 		api = AUDIO_NONE;
 		
@@ -1690,6 +1686,7 @@ void audio_api_changed(GtkComboBox *combo, void *data)
 		gtk_widget_set_sensitive(my_audio_widgets->device, FALSE);
 		gtk_widget_set_sensitive(my_audio_widgets->channels, FALSE);
 		gtk_widget_set_sensitive(my_audio_widgets->samprate, FALSE);
+		gtk_widget_set_sensitive(my_audio_widgets->latency, FALSE);
 	}
 	else
 	{
@@ -1720,23 +1717,45 @@ void audio_api_changed(GtkComboBox *combo, void *data)
 		gtk_widget_set_sensitive (my_audio_widgets->device, TRUE);
 		gtk_widget_set_sensitive(my_audio_widgets->channels, TRUE);
 		gtk_widget_set_sensitive(my_audio_widgets->samprate, TRUE);
+		gtk_widget_set_sensitive(my_audio_widgets->latency, TRUE);
 
 		/*update channels*/
 		int index = gtk_combo_box_get_active(GTK_COMBO_BOX(my_audio_widgets->channels));
 
-		if(index == 0)
+		if(index == 0) /*auto*/
 			audio_ctx->channels = audio_ctx->list_devices[audio_ctx->device].channels;
 
 		/*update samprate*/
 		index = gtk_combo_box_get_active(GTK_COMBO_BOX(my_audio_widgets->samprate));
 
-		if(index == 0)
+		if(index == 0) /*auto*/
 			audio_ctx->samprate = audio_ctx->list_devices[audio_ctx->device].samprate;
-
+		
+		gtk_range_set_value(GTK_RANGE(my_audio_widgets->latency), audio_ctx->latency);	
 	}
 
 }
 
+/*
+ * audio latency changed event
+ * args:
+ *    range - widget that generated the event
+ *    data - pointer to user data
+ *
+ * asserts:
+ *    none
+ *
+ * returns: none
+ */
+void audio_latency_changed(GtkRange *range, void *data)
+{
+	/**update the audio context for the new api*/
+	audio_context_t *audio_ctx = get_audio_context();
+
+	if(audio_ctx != NULL)
+		audio_ctx->latency = (double) gtk_range_get_value (range);
+}
+
 /*
  * video encoder properties clicked event
  * args:
@@ -1763,7 +1782,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	GtkWidget *table = gtk_grid_new();
 
 	GtkWidget *lbl_fps = gtk_label_new(_("                              encoder fps:   \n (0 - use fps combobox value)"));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_fps), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_fps), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_fps), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_fps, 0, line, 1, 1);
 	gtk_widget_show (lbl_fps);
 
@@ -1783,7 +1807,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_bit_rate = gtk_label_new(_("bit rate:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_bit_rate), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_bit_rate), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_bit_rate), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_bit_rate, 0, line, 1, 1);
 	gtk_widget_show (lbl_bit_rate);
 
@@ -1796,7 +1825,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_qmax = gtk_label_new(_("qmax:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_qmax), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_qmax), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_qmax), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_qmax, 0, line, 1 ,1);
 	gtk_widget_show (lbl_qmax);
 
@@ -1809,7 +1843,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_qmin = gtk_label_new(_("qmin:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_qmin), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_qmin), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_qmin), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_qmin, 0, line, 1, 1);
 	gtk_widget_show (lbl_qmin);
 
@@ -1822,7 +1861,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_max_qdiff = gtk_label_new(_("max. qdiff:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_max_qdiff), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_max_qdiff), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_max_qdiff), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_max_qdiff, 0, line, 1, 1);
 	gtk_widget_show (lbl_max_qdiff);
 
@@ -1835,7 +1879,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_dia = gtk_label_new(_("dia size:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_dia), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_dia), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_dia), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_dia, 0, line, 1, 1);
 	gtk_widget_show (lbl_dia);
 
@@ -1848,7 +1897,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_pre_dia = gtk_label_new(_("pre dia size:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_pre_dia), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_pre_dia), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_pre_dia), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_pre_dia, 0, line, 1, 1);
 	gtk_widget_show (lbl_pre_dia);
 
@@ -1861,7 +1915,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_pre_me = gtk_label_new(_("pre me:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_pre_me), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_pre_me), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_pre_me), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_pre_me, 0, line, 1, 1);
 	gtk_widget_show (lbl_pre_me);
 
@@ -1874,7 +1933,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_me_pre_cmp = gtk_label_new(_("pre cmp:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_me_pre_cmp), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_me_pre_cmp), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_me_pre_cmp), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_me_pre_cmp, 0, line, 1, 1);
 	gtk_widget_show (lbl_me_pre_cmp);
 
@@ -1887,7 +1951,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_me_cmp = gtk_label_new(_("cmp:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_me_cmp), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_me_cmp), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_me_cmp), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_me_cmp, 0, line, 1, 1);
 	gtk_widget_show (lbl_me_cmp);
 
@@ -1900,7 +1969,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_me_sub_cmp = gtk_label_new(_("sub cmp:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_me_sub_cmp), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_me_sub_cmp), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_me_sub_cmp), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_me_sub_cmp, 0, line, 1, 1);
 	gtk_widget_show (lbl_me_sub_cmp);
 
@@ -1913,7 +1987,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_last_pred = gtk_label_new(_("last predictor count:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_last_pred), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_last_pred), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_last_pred), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_last_pred, 0, line, 1, 1);
 	gtk_widget_show (lbl_last_pred);
 
@@ -1926,7 +2005,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_gop_size = gtk_label_new(_("gop size:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_gop_size), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_gop_size), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_gop_size), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_gop_size, 0, line, 1, 1);
 	gtk_widget_show (lbl_gop_size);
 
@@ -1939,7 +2023,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_qcompress = gtk_label_new(_("qcompress:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_qcompress), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_qcompress), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_qcompress), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_qcompress, 0, line, 1, 1);
 	gtk_widget_show (lbl_qcompress);
 
@@ -1952,7 +2041,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_qblur = gtk_label_new(_("qblur:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_qblur), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_qblur), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_qblur), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_qblur, 0, line, 1 ,1);
 	gtk_widget_show (lbl_qblur);
 
@@ -1965,7 +2059,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_subq = gtk_label_new(_("subq:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_subq), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_subq), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_subq), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_subq, 0, line, 1 ,1);
 	gtk_widget_show (lbl_subq);
 
@@ -1978,7 +2077,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_framerefs = gtk_label_new(_("framerefs:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_framerefs), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_framerefs), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_framerefs), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_framerefs, 0, line, 1 ,1);
 	gtk_widget_show (lbl_framerefs);
 
@@ -1991,7 +2095,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_me_method = gtk_label_new(_("me method:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_me_method), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_me_method), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_me_method), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_me_method, 0, line, 1 ,1);
 	gtk_widget_show (lbl_me_method);
 
@@ -2004,7 +2113,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_mb_decision = gtk_label_new(_("mb decision:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_mb_decision), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_mb_decision), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_mb_decision), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_mb_decision, 0, line, 1 ,1);
 	gtk_widget_show (lbl_mb_decision);
 
@@ -2017,7 +2131,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_max_b_frames = gtk_label_new(_("max B frames:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_max_b_frames), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_max_b_frames), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_max_b_frames), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_max_b_frames, 0, line, 1 ,1);
 	gtk_widget_show (lbl_max_b_frames);
 
@@ -2030,7 +2149,12 @@ void encoder_video_properties(GtkMenuItem *item, void *data)
 	line++;
 
 	GtkWidget *lbl_num_threads = gtk_label_new(_("num threads:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_num_threads), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_num_threads), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_num_threads), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_num_threads, 0, line, 1 ,1);
 	gtk_widget_show (lbl_num_threads);
 
@@ -2108,7 +2232,12 @@ void encoder_audio_properties(GtkMenuItem *item, void *data)
 
 	/*bit rate*/
 	GtkWidget *lbl_bit_rate = gtk_label_new(_("bit rate:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_bit_rate), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_bit_rate), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_bit_rate), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_bit_rate, 0, line, 1, 1);
 	gtk_widget_show (lbl_bit_rate);
 
@@ -2122,7 +2251,12 @@ void encoder_audio_properties(GtkMenuItem *item, void *data)
 
 	/*sample format*/
 	GtkWidget *lbl_sample_fmt = gtk_label_new(_("sample format:   "));
+#if GTK_VER_AT_LEAST(3,15)
+	gtk_label_set_xalign(GTK_LABEL(lbl_sample_fmt), 1);
+	gtk_label_set_yalign(GTK_LABEL(lbl_sample_fmt), 0.5);
+#else
 	gtk_misc_set_alignment (GTK_MISC (lbl_sample_fmt), 1, 0.5);
+#endif
 	gtk_grid_attach (GTK_GRID(table), lbl_sample_fmt, 0, line, 1, 1);
 	gtk_widget_show (lbl_sample_fmt);
 
@@ -2160,17 +2294,12 @@ void encoder_audio_properties(GtkMenuItem *item, void *data)
  *   data - pointer to user data
  *
  * asserts:
- *   data (device) is not NULL
+ *   none
  *
  * returns: true if we handled the event or false otherwise
  */
 gboolean window_key_pressed (GtkWidget *win, GdkEventKey *event, void *data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
-	/*assertions*/
-	assert(device != NULL);
-
 	/* If we have modifiers, and either Ctrl, Mod1 (Alt), or any
 	 * of Mod3 to Mod5 (Mod2 is num-lock...) are pressed, we
 	 * let Gtk+ handle the key */
@@ -2183,7 +2312,7 @@ gboolean window_key_pressed (GtkWidget *win, GdkEventKey *event, void *data)
 		|| (event->state & GDK_MOD5_MASK)))
 		return FALSE;
 
-    if(device->has_pantilt_control_id)
+    if(v4l2core_has_pantilt_id())
     {
 		int id = 0;
 		int value = 0;
@@ -2193,25 +2322,25 @@ gboolean window_key_pressed (GtkWidget *win, GdkEventKey *event, void *data)
             case GDK_KEY_Down:
             case GDK_KEY_KP_Down:
 				id = V4L2_CID_TILT_RELATIVE;
-				value = device->tilt_step;
+				value = v4l2core_get_tilt_step();
 				break;
 
             case GDK_KEY_Up:
             case GDK_KEY_KP_Up:
 				id = V4L2_CID_TILT_RELATIVE;
-				value = - device->tilt_step;
+				value = - v4l2core_get_tilt_step();
 				break;
 
             case GDK_KEY_Left:
             case GDK_KEY_KP_Left:
 				id = V4L2_CID_PAN_RELATIVE;
-				value = device->pan_step;
+				value = v4l2core_get_pan_step();
 				break;
 
             case GDK_KEY_Right:
             case GDK_KEY_KP_Right:
                 id = V4L2_CID_PAN_RELATIVE;
-				value = - device->pan_step;
+				value = - v4l2core_get_pan_step();
 				break;
 
             default:
@@ -2220,13 +2349,13 @@ gboolean window_key_pressed (GtkWidget *win, GdkEventKey *event, void *data)
 
         if(id != 0 && value != 0)
         {
-			v4l2_ctrl_t *control = v4l2core_get_control_by_id(device, id);
+			v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
 
 			if(control)
 			{
 				control->value =  value;
 
-				if(v4l2core_set_control_value_by_id(device, id))
+				if(v4l2core_set_control_value_by_id(id))
 					fprintf(stderr, "GUVCVIEW: error setting pan/tilt value\n");
 
 				return TRUE;
@@ -2265,22 +2394,17 @@ gboolean window_key_pressed (GtkWidget *win, GdkEventKey *event, void *data)
  *   data - pointer to user data
  *
  * asserts:
- *   data (device) is not NULL
+ *   none
  *
  * returns: true if timer is to be reset or false otherwise
  */
 gboolean check_device_events(gpointer data)
 {
-	v4l2_dev_t *device = (v4l2_dev_t *) data;
-
-	/*assertions*/
-	assert(device != NULL);
-
-	if(v4l2core_check_device_list_events(device))
+	if(v4l2core_check_device_list_events())
 	{
 		/*update device list*/
 		g_signal_handlers_block_by_func(GTK_COMBO_BOX_TEXT(get_wgtDevices_gtk3()),
-                G_CALLBACK (devices_changed), device);
+                G_CALLBACK (devices_changed), NULL);
 
 		GtkListStore *store = GTK_LIST_STORE(gtk_combo_box_get_model (GTK_COMBO_BOX(get_wgtDevices_gtk3())));
 		gtk_list_store_clear(store);
@@ -2296,7 +2420,7 @@ gboolean check_device_events(gpointer data)
 		}
 
 		g_signal_handlers_unblock_by_func(GTK_COMBO_BOX_TEXT(get_wgtDevices_gtk3()),
-                G_CALLBACK (devices_changed), device);
+                G_CALLBACK (devices_changed), NULL);
 	}
 
 	return (TRUE);
