@@ -93,19 +93,16 @@ control_widgets_t *gui_gtk3_get_widgets_by_id(int id)
 /*
  * attach v4l2 controls tab widget
  * args:
- *   device - pointer to device data we want to attach the gui for
  *   parent - tab parent widget
  *
  * asserts:
- *   device is not null
  *   parent is not null
  *
  * returns: error code (0 -OK)
  */
-int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
+int gui_attach_gtk3_v4l2ctrls(GtkWidget *parent)
 {
 	/*assertions*/
-	assert(device != NULL);
 	assert(parent != NULL);
 
 	if(debug_level > 1)
@@ -124,10 +121,16 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 	int i = 0;
 	int n = 0;
-	v4l2_ctrl_t *current = device->list_device_controls;
+	v4l2_ctrl_t *current = v4l2core_get_control_list();
 
     for(; current != NULL; current = current->next, ++n)
     {
+		if(current == NULL)
+		{
+			fprintf(stderr, "GUVCVIEW: ERROR (attach gtk3 controls) empty control in list\n");
+			break;
+		}
+
 		if(!is_control_panel &&
 		   (current->control.id == V4L2_CID_FOCUS_LOGITECH ||
 		    current->control.id == V4L2_CID_FOCUS_ABSOLUTE))
@@ -149,7 +152,13 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
         control_widgets_list[widget_list_size - 1].label = gtk_label_new (tmp);
         g_free(tmp);
         gtk_widget_show (control_widgets_list[widget_list_size - 1].label);
-        gtk_misc_set_alignment (GTK_MISC (control_widgets_list[widget_list_size - 1].label), 1, 0.5);
+#if GTK_VER_AT_LEAST(3,15)
+				gtk_label_set_xalign(GTK_LABEL(control_widgets_list[widget_list_size - 1].label), 1);
+				gtk_label_set_yalign(GTK_LABEL(control_widgets_list[widget_list_size - 1].label), 0.5);
+#else
+				gtk_misc_set_alignment (GTK_MISC (control_widgets_list[widget_list_size - 1].label), 1, 0.5);
+#endif
+        
 
 		control_widgets_list[widget_list_size - 1].id = current->control.id;
         control_widgets_list[widget_list_size - 1].widget = NULL;
@@ -192,9 +201,9 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 						/*connect signals*/
 						g_signal_connect (GTK_BUTTON(PanTilt1), "clicked",
-							G_CALLBACK (button_PanTilt1_clicked), device);
+							G_CALLBACK (button_PanTilt1_clicked), NULL);
 						g_signal_connect (GTK_BUTTON(PanTilt2), "clicked",
-							G_CALLBACK (button_PanTilt2_clicked), device);
+							G_CALLBACK (button_PanTilt2_clicked), NULL);
 
 						gtk_widget_show (control_widgets_list[n].widget);
 
@@ -203,15 +212,15 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 						gtk_editable_set_editable(GTK_EDITABLE(control_widgets_list[n].widget2), TRUE);
 
 						if(current->control.id == V4L2_CID_PAN_RELATIVE)
-							gtk_spin_button_set_value (GTK_SPIN_BUTTON(control_widgets_list[n].widget2), device->pan_step);
+							gtk_spin_button_set_value (GTK_SPIN_BUTTON(control_widgets_list[n].widget2), v4l2core_get_pan_step());
 						else
-							gtk_spin_button_set_value (GTK_SPIN_BUTTON(control_widgets_list[n].widget2), device->tilt_step);
+							gtk_spin_button_set_value (GTK_SPIN_BUTTON(control_widgets_list[n].widget2),v4l2core_get_tilt_step());
 
 						/*connect signal*/
 						g_object_set_data (G_OBJECT (control_widgets_list[n].widget2), "control_info",
 							GINT_TO_POINTER(current->control.id));
 						g_signal_connect(GTK_SPIN_BUTTON(control_widgets_list[n].widget2),"value-changed",
-							G_CALLBACK (pan_tilt_step_changed), device);
+							G_CALLBACK (pan_tilt_step_changed), NULL);
 
 						gtk_widget_show (control_widgets_list[n].widget2);
 
@@ -229,7 +238,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 						/*connect signal*/
 						g_signal_connect (GTK_BUTTON(control_widgets_list[n].widget), "clicked",
-							G_CALLBACK (button_clicked), device);
+							G_CALLBACK (button_clicked), NULL);
 
 						break;
 					};
@@ -285,7 +294,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 						/*connect signal*/
 						g_signal_connect (GTK_COMBO_BOX_TEXT(control_widgets_list[n].widget), "changed",
-							G_CALLBACK (combo_changed), device);
+							G_CALLBACK (combo_changed), NULL);
 
 						break;
 					}
@@ -335,7 +344,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 							GINT_TO_POINTER(current->control.id));
 						/*connect signal*/
 						g_signal_connect (GTK_COMBO_BOX_TEXT(control_widgets_list[n].widget), "changed",
-							G_CALLBACK (combo_changed), device);
+							G_CALLBACK (combo_changed), NULL);
 						break;
 					}
 
@@ -353,12 +362,11 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 
 							gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (control_widgets_list[n-1].widget), FALSE);
-								//device->aux_flag2 ? TRUE: FALSE);
 
 							g_signal_connect (G_OBJECT (control_widgets_list[n-1].widget), "toggled",
-								G_CALLBACK (autofocus_changed), device);
+								G_CALLBACK (autofocus_changed), NULL);
 							g_signal_connect (G_OBJECT (control_widgets_list[n-1].widget2), "clicked",
-								G_CALLBACK (setfocus_clicked), device);
+								G_CALLBACK (setfocus_clicked), NULL);
 
 							gtk_grid_attach(GTK_GRID(img_controls_grid), control_widgets_list[n-1].widget, 1, i, 1 , 1);
 							gtk_widget_set_halign (control_widgets_list[n-1].widget, GTK_ALIGN_FILL);
@@ -384,7 +392,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 							control_widgets_list[n].widget = gtk_scale_new (GTK_ORIENTATION_HORIZONTAL, adjustment);
 
 							gtk_scale_set_draw_value (GTK_SCALE (control_widgets_list[n].widget), FALSE);
-							gtk_range_set_round_digits(GTK_RANGE (control_widgets_list[n].widget), 0);
+							gtk_scale_set_digits(GTK_SCALE(control_widgets_list[n].widget), 0);
 
 							gtk_widget_show (control_widgets_list[n].widget);
 
@@ -411,9 +419,9 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 							/*connect signals*/
 							g_signal_connect (GTK_SCALE(control_widgets_list[n].widget), "value-changed",
-								G_CALLBACK (slider_changed), device);
+								G_CALLBACK (slider_changed), NULL);
 							g_signal_connect(GTK_SPIN_BUTTON(control_widgets_list[n].widget2),"value-changed",
-								G_CALLBACK (spin_changed), device);
+								G_CALLBACK (spin_changed), NULL);
 						}
 						else
                           fprintf(stderr, "GUVCVIEW: (Invalid range) [MAX <= MIN] for control id: 0x%08x \n", current->control.id);
@@ -442,7 +450,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 				/*connect signal*/
 				g_signal_connect (GTK_BUTTON(control_widgets_list[n].widget2), "clicked",
-					G_CALLBACK (int64_button_clicked), device);
+					G_CALLBACK (int64_button_clicked), NULL);
 
 				break;
 #endif
@@ -465,7 +473,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 				/*connect signal*/
 				g_signal_connect (GTK_BUTTON(control_widgets_list[n].widget2), "clicked",
-					G_CALLBACK (string_button_clicked), device);
+					G_CALLBACK (string_button_clicked), NULL);
 
 				break;
 #endif
@@ -486,7 +494,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 						widget);
 
                     g_signal_connect (GTK_BUTTON(control_widgets_list[n].widget2), "clicked",
-                        G_CALLBACK (bitmask_button_clicked), device);
+                        G_CALLBACK (bitmask_button_clicked), NULL);
 
 				break;
 #endif
@@ -530,7 +538,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 					/*connect signal*/
 					g_signal_connect (GTK_COMBO_BOX_TEXT(control_widgets_list[n].widget), "changed",
-						G_CALLBACK (combo_changed), device);
+						G_CALLBACK (combo_changed), NULL);
 				}
                 break;
 
@@ -543,7 +551,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 					GINT_TO_POINTER(current->control.id));
 
 				g_signal_connect (GTK_BUTTON(control_widgets_list[n].widget), "clicked",
-					G_CALLBACK (button_clicked), device);
+					G_CALLBACK (button_clicked), NULL);
                 break;
 
             case V4L2_CTRL_TYPE_BOOLEAN:
@@ -561,16 +569,18 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 					gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(control_widgets_list[n].widget2),
 						"RGRG... | GBGB...");
 
-					device->bayer_pix_order = 0;
-					gtk_combo_box_set_active(GTK_COMBO_BOX(control_widgets_list[n].widget2), device->bayer_pix_order);
+					v4l2core_set_bayer_pix_order(0);
+					
+					gtk_combo_box_set_active(GTK_COMBO_BOX(control_widgets_list[n].widget2), v4l2core_get_bayer_pix_order());
 
 					gtk_widget_show (control_widgets_list[n].widget2);
 
 					/*connect signal*/
 					g_signal_connect (GTK_COMBO_BOX_TEXT (control_widgets_list[n].widget2), "changed",
-						G_CALLBACK (bayer_pix_ord_changed), device);
+						G_CALLBACK (bayer_pix_ord_changed), NULL);
 
-					device->isbayer = (current->value ? TRUE : FALSE);
+					uint8_t isbayer = (current->value ? TRUE : FALSE);
+					v4l2core_set_isbayer(isbayer);
 				}
 
 				control_widgets_list[n].widget = gtk_check_button_new();
@@ -583,7 +593,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 
 				/*connect signal*/
 				g_signal_connect (GTK_TOGGLE_BUTTON(control_widgets_list[n].widget), "toggled",
-					G_CALLBACK (check_changed), device);
+					G_CALLBACK (check_changed), NULL);
 
                 break;
 
@@ -614,7 +624,7 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 	/*add control grid to parent container*/
 	gtk_container_add(GTK_CONTAINER(parent), img_controls_grid);
 
-	gui_gtk3_update_controls_state(device);
+	gui_gtk3_update_controls_state();
 
 	return 0;
 }
@@ -622,22 +632,25 @@ int gui_attach_gtk3_v4l2ctrls(v4l2_dev_t *device, GtkWidget *parent)
 /*
  * update the controls widgets state
  * args:
- *   device - pointer to device data
+ *   none
  *
  * asserts:
- *   device is not null
+ *   none
  *
  * returns: none
  */
-void gui_gtk3_update_controls_state(v4l2_dev_t *device)
+void gui_gtk3_update_controls_state()
 {
-	/*asserts*/
-	assert(device != NULL);
-
-	v4l2_ctrl_t *current = device->list_device_controls;
+	v4l2_ctrl_t *current = v4l2core_get_control_list();
 
     for(; current != NULL; current = current->next)
     {
+		if(current == NULL)
+		{
+			fprintf(stderr, "GUVCVIEW: ERROR (update controls state) empty control in list\n");
+			break;
+		}
+
 		control_widgets_t *cur_widget = gui_gtk3_get_widgets_by_id(current->control.id);
 
 		if(!cur_widget)
@@ -661,12 +674,12 @@ void gui_gtk3_update_controls_state(v4l2_dev_t *device)
 			case V4L2_CTRL_TYPE_BOOLEAN:
 				/*disable widget signals*/
 				g_signal_handlers_block_by_func(GTK_TOGGLE_BUTTON(cur_widget->widget),
-					G_CALLBACK (check_changed), device);
+					G_CALLBACK (check_changed), NULL);
 				gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (cur_widget->widget),
 					current->value ? TRUE : FALSE);
 				/*enable widget signals*/
 				g_signal_handlers_unblock_by_func(GTK_TOGGLE_BUTTON(cur_widget->widget),
-					G_CALLBACK (check_changed), device);
+					G_CALLBACK (check_changed), NULL);
 				break;
 
 #ifdef V4L2_CTRL_TYPE_BITMASK
@@ -699,21 +712,21 @@ void gui_gtk3_update_controls_state(v4l2_dev_t *device)
 				{
 					/*disable widget signals*/
 					g_signal_handlers_block_by_func(GTK_SCALE (cur_widget->widget),
-						G_CALLBACK (slider_changed), device);
+						G_CALLBACK (slider_changed), NULL);
 					gtk_range_set_value (GTK_RANGE (cur_widget->widget), current->value);
 					/*enable widget signals*/
 					g_signal_handlers_unblock_by_func(GTK_SCALE (cur_widget->widget),
-						G_CALLBACK (slider_changed), device);
+						G_CALLBACK (slider_changed), NULL);
 
 					if(cur_widget->widget2)
 					{
 						/*disable widget signals*/
 						g_signal_handlers_block_by_func(GTK_SPIN_BUTTON(cur_widget->widget2),
-							G_CALLBACK (spin_changed), device);
+							G_CALLBACK (spin_changed), NULL);
 						gtk_spin_button_set_value (GTK_SPIN_BUTTON(cur_widget->widget2), current->value);
 						/*enable widget signals*/
 						g_signal_handlers_unblock_by_func(GTK_SPIN_BUTTON(cur_widget->widget2),
-							G_CALLBACK (spin_changed), device);
+							G_CALLBACK (spin_changed), NULL);
 					}
 				}
 				break;
@@ -725,7 +738,7 @@ void gui_gtk3_update_controls_state(v4l2_dev_t *device)
 			{
 				/*disable widget signals*/
 				g_signal_handlers_block_by_func(GTK_COMBO_BOX_TEXT(cur_widget->widget),
-					G_CALLBACK (combo_changed), device);
+					G_CALLBACK (combo_changed), NULL);
 				/*get new index*/
 				int j = 0;
 				int def = 0;
@@ -738,7 +751,7 @@ void gui_gtk3_update_controls_state(v4l2_dev_t *device)
 				gtk_combo_box_set_active(GTK_COMBO_BOX(cur_widget->widget), def);
 				/*enable widget signals*/
 				g_signal_handlers_unblock_by_func(GTK_COMBO_BOX_TEXT(cur_widget->widget),
-					G_CALLBACK (combo_changed), device);
+					G_CALLBACK (combo_changed), NULL);
 				break;
 			}
 
