@@ -1348,19 +1348,11 @@ void jpeg_close_decoder()
 #else  //use libavcodec to decode mjpeg data
 
 /*h264 decoder (libavcodec)*/
-#ifdef HAVE_AVCODEC_H
-  #include <avcodec.h>
+#ifdef HAVE_FFMPEG_AVCODEC_H
+#include <ffmpeg/avcodec.h>
 #else
-  #ifdef HAVE_LIBAVCODEC_AVCODEC_H
-    #include <libavcodec/avcodec.h>
-  #else
-    #ifdef HAVE_FFMPEG_AVCODEC_H
-      #include <ffmpeg/avcodec.h>
-    #else
-      #include <libavcodec/avcodec.h>
-    #endif
-  #endif
-#endif
+#include <libavcodec/avcodec.h>
+#endif 
 
 #define LIBAVCODEC_VER_AT_LEAST(major,minor)  (LIBAVCODEC_VERSION_MAJOR > major || \
                                               (LIBAVCODEC_VERSION_MAJOR == major && \
@@ -1369,6 +1361,11 @@ void jpeg_close_decoder()
 #if !LIBAVCODEC_VER_AT_LEAST(54,25)
 	#define AV_CODEC_ID_H264 CODEC_ID_H264
 #endif
+
+#if !LIBAVCODEC_VER_AT_LEAST(54,25)
+	#define AV_CODEC_ID_MJPEG CODEC_ID_MJPEG
+#endif
+
 typedef struct _codec_data_t
 {
 	AVCodec *codec;
