@@ -146,7 +146,7 @@ void v4l2core_soft_autofocus_set_sort(int method)
  *
  * returns: error code (0 - E_OK)
  */
-int v4l2core_soft_autofocus_init (v4l2_dev_t *vd)
+int soft_autofocus_init (v4l2_dev_t *vd)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -167,7 +167,7 @@ int v4l2core_soft_autofocus_init (v4l2_dev_t *vd)
 		exit(-1);
 	}
 
-    focus_ctx->focus_control = v4l2core_get_control_by_id(vd, vd->has_focus_control_id);
+    focus_ctx->focus_control = v4l2core_get_control_by_id(vd->has_focus_control_id);
 
     if(focus_ctx->focus_control == NULL)
 	{
@@ -762,6 +762,7 @@ int soft_autofocus_get_focus_value()
  * run the software autofocus
  * args:
  *    vd - pointer to device data
+ *    frame - pointer to frame buffer
  *
  * asserts:
  *    vd is not null
@@ -769,7 +770,7 @@ int soft_autofocus_get_focus_value()
  * returns: 1 - running  0- focused
  * 	(only matters for non-continue focus)
  */
-int v4l2core_soft_autofocus_run(v4l2_dev_t *vd)
+int soft_autofocus_run(v4l2_dev_t *vd, v4l2_frame_buff_t *frame)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -780,7 +781,7 @@ int v4l2core_soft_autofocus_run(v4l2_dev_t *vd)
 		focus_ctx->focus = focus_ctx->left; /*start left*/
 
 		focus_ctx->focus_control->value = focus_ctx->focus;
-		if (v4l2core_set_control_value_by_id(vd, focus_ctx->focus_control->control.id) != 0)
+		if (v4l2core_set_control_value_by_id(focus_ctx->focus_control->control.id) != 0)
 			fprintf(stderr, "V4L2_CORE: (sof_autofocus) couldn't set focus to %d\n", focus_ctx->focus);
 
 		/*number of frames until focus is stable*/
@@ -793,7 +794,7 @@ int v4l2core_soft_autofocus_run(v4l2_dev_t *vd)
 		if (focus_ctx->focus_wait == 0)
 		{
 			focus_ctx->sharpness = soft_autofocus_get_sharpness (
-				vd->yuv_frame,
+				frame->yuv_frame,
 				vd->format.fmt.pix.width,
 				vd->format.fmt.pix.height,
 				5);
@@ -813,7 +814,7 @@ int v4l2core_soft_autofocus_run(v4l2_dev_t *vd)
 			if ((focus_ctx->focus != focus_ctx->last_focus))
 			{
 				focus_ctx->focus_control->value = focus_ctx->focus;
-				if (v4l2core_set_control_value_by_id(vd, focus_ctx->focus_control->control.id) != 0)
+				if (v4l2core_set_control_value_by_id(focus_ctx->focus_control->control.id) != 0)
 					fprintf(stderr, "V4L2_CORE: (sof_autofocus) couldn't set focus to %d\n",
 						focus_ctx->focus);
 
