@@ -24,7 +24,6 @@
 /* support for internationalization - i18n */
 #include <inttypes.h>
 #include <math.h>
-#include <libusb.h>
 #include <stdlib.h>
 #include <stdio.h>
 #include <linux/videodev2.h>
@@ -62,10 +61,6 @@
 #include "uvc_h264.h"
 #include "v4l2_formats.h"
 
-#define USB_VIDEO_CONTROL		    0x01
-#define USB_VIDEO_CONTROL_INTERFACE	0x24
-#define USB_VIDEO_CONTROL_XU_TYPE	0x06
-
 // GUID of the UVC H.264 extension unit: {A29E7641-DE04-47E3-8B2B-F4341AFF003B}
 #define GUID_UVCX_H264_XU {0x41, 0x76, 0x9E, 0xA2, 0x04, 0xDE, 0xE3, 0x47, 0x8B, 0x2B, 0xF4, 0x34, 0x1A, 0xFF, 0x00, 0x3B}
 
@@ -98,7 +93,7 @@ static int h264_support = H264_NONE; /*none by default*/
  *
  * returns: none
  */
-void v4l2core_h264_request_idr(v4l2_dev_t *vd)
+void h264_request_idr(v4l2_dev_t *vd)
 {
 	request_h264_frame_type(vd, PICTURE_TYPE_IDR_FULL);
 }
@@ -184,7 +179,7 @@ static int uvcx_video_encoder_reset(v4l2_dev_t *vd)
 
 	int err = 0;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_ENCODER_RESET, UVC_SET_CUR, &encoder_reset_req)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_ENCODER_RESET, UVC_SET_CUR, &encoder_reset_req)) < 0)
 		fprintf(stderr, "V4L2_CORE: (UVCX_ENCODER_RESET) error: %s\n", strerror(errno));
 
 	return err;
@@ -210,7 +205,7 @@ static int uvcx_video_probe(v4l2_dev_t *vd, uint8_t query, uvcx_video_config_pro
 	int err = 0;
 
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_VIDEO_CONFIG_PROBE, query, uvcx_video_config)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_VIDEO_CONFIG_PROBE, query, uvcx_video_config)) < 0)
 		fprintf(stderr, "V4L2_CORE: (UVCX_VIDEO_CONFIG_PROBE) error: %s\n", strerror(errno));
 
 	return err;
@@ -234,7 +229,7 @@ static int uvcx_video_commit(v4l2_dev_t *vd, uvcx_video_config_probe_commit_t *u
 
 	int err = 0;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_VIDEO_CONFIG_COMMIT, UVC_SET_CUR, uvcx_video_config)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_VIDEO_CONFIG_COMMIT, UVC_SET_CUR, uvcx_video_config)) < 0)
 		fprintf(stderr, "V4L2_CORE: (UVCX_VIDEO_CONFIG_COMMIT) error: %s\n", strerror(errno));
 
 	return err;
@@ -255,109 +250,12 @@ static int uvcx_video_commit(v4l2_dev_t *vd, uvcx_video_config_probe_commit_t *u
  */
 uint8_t get_uvc_h624_unit_id (v4l2_dev_t *vd)
 {
-	v4l2_device_list *my_device_list = v4l2core_get_device_list();
-
-	/*asserts*/
-	assert(vd != NULL);
-	assert(my_device_list->list_devices != NULL);
-
-	uint64_t busnum = my_device_list->list_devices[vd->this_device].busnum;
-	uint64_t devnum = my_device_list->list_devices[vd->this_device].devnum;
-
-	if(verbosity > 2)
-		printf("V4L2_CORE: checking h264 unit id for device %i (bus:%"PRId64" dev:%"PRId64")\n", vd->this_device, busnum, devnum);
-    /* use libusb */
-	libusb_context *usb_ctx = NULL;
-    libusb_device **device_list = NULL;
-    libusb_device *device = NULL;
-    ssize_t cnt;
-    int i;
-
-	static const uint8_t guid[16] = GUID_UVCX_H264_XU;
-	vd->h264_unit_id = 0;/*reset it*/
-
-    if (usb_ctx == NULL)
-      libusb_init (&usb_ctx);
-
-    cnt = libusb_get_device_list (usb_ctx, &device_list);
-    for (i = 0; i < cnt; i++)
-    {
-		uint64_t dev_busnum = libusb_get_bus_number (device_list[i]);
-		uint64_t dev_devnum = libusb_get_device_address (device_list[i]);
-
-		if(verbosity > 2)
-			printf("V4L2_CORE: (libusb) checking bus(%" PRId64 ") dev(%" PRId64 ") for device\n", dev_busnum, dev_devnum);
-
-		if (busnum == dev_busnum &&	devnum == dev_devnum)
-		{
-			device = libusb_ref_device (device_list[i]);
-			break;
-		}
-	}
-
-	libusb_free_device_list (device_list, 1);
-
-	if (device)
-	{
-		if(verbosity > 1)
-			printf("V4L2_CORE: (libusb) checking for H264 unit id\n");
-		struct libusb_device_descriptor desc;
-
-		 if (libusb_get_device_descriptor (device, &desc) == 0)
-		 {
-			for (i = 0; i < desc.bNumConfigurations; ++i)
-			{
-				struct libusb_config_descriptor *config = NULL;
-
-				if (libusb_get_config_descriptor (device, i, &config) == 0)
-				{
-					int j = 0;
-					for (j = 0; j < config->bNumInterfaces; j++)
-					{
-						int k = 0;
-						for (k = 0; k < config->interface[j].num_altsetting; k++)
-						{
-							const struct libusb_interface_descriptor *interface;
-							const uint8_t *ptr = NULL;
-
-							interface = &config->interface[j].altsetting[k];
-							if (interface->bInterfaceClass != LIBUSB_CLASS_VIDEO ||
-								interface->bInterfaceSubClass != USB_VIDEO_CONTROL)
-								continue;
-							ptr = interface->extra;
-							while (ptr - interface->extra +
-								sizeof (xu_descriptor) < interface->extra_length)
-							{
-								xu_descriptor *desc = (xu_descriptor *) ptr;
-
-								if (desc->bDescriptorType == USB_VIDEO_CONTROL_INTERFACE &&
-									desc->bDescriptorSubType == USB_VIDEO_CONTROL_XU_TYPE &&
-									memcmp (desc->guidExtensionCode, guid, 16) == 0)
-								{
-									vd->h264_unit_id = desc->bUnitID;
-
-									libusb_unref_device (device);
-									/*it's a match*/
-									if(verbosity > 1)
-										printf("V4L2_CORE: (libusb) found H264 unit id %i\n", vd->h264_unit_id);
-									return vd->h264_unit_id;
-								}
-								ptr += desc->bLength;
-							}
-						}
-					}
-				}
-				else
-					fprintf(stderr, "V4L2_CORE: (libusb) couldn't get config descriptor for configuration %i\n", i);
-			}
-		}
-		else
-			fprintf(stderr, "V4L2_CORE: (libusb) couldn't get device descriptor\n");
-		libusb_unref_device (device);
-	}
-	else
-		fprintf(stderr, "V4L2_CORE: (libusb) couldn't get device\n");
-	/*no match found*/
+	if(verbosity > 1)
+		printf("V4L2_CORE: checking for UVCX_H264 unit id\n");
+	
+	uint8_t guid[16] = GUID_UVCX_H264_XU;	
+	vd->h264_unit_id = get_guid_unit_id (vd, guid);
+	
 	return vd->h264_unit_id;
 }
 
@@ -389,7 +287,7 @@ int check_h264_support(v4l2_dev_t *vd)
 
 	uvcx_version_t uvcx_version;
 
-	if(v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_VERSION, UVC_GET_CUR, &uvcx_version) < 0)
+	if(v4l2core_query_xu_control(vd->h264_unit_id, UVCX_VERSION, UVC_GET_CUR, &uvcx_version) < 0)
 	{
 		if(verbosity > 0)
 			printf("V4L2_CORE: device doesn't seem to support uvc H264 in unit_id %d\n", vd->h264_unit_id);
@@ -422,7 +320,7 @@ void add_h264_format(v4l2_dev_t *vd)
 	if(verbosity > 0)
 		printf("V4L2_CORE: checking muxed H264 format support\n");
 
-	if(v4l2core_get_frame_format_index(vd, V4L2_PIX_FMT_H264) >= 0)
+	if(v4l2core_get_frame_format_index(V4L2_PIX_FMT_H264) >= 0)
 	{
 		if(verbosity > 0)
 			printf("V4L2_CORE: H264 format already in list\n");
@@ -446,7 +344,7 @@ void add_h264_format(v4l2_dev_t *vd)
 		return; /*no XU support for h264*/
 	}
 
-	int mjpg_index = v4l2core_get_frame_format_index(vd, V4L2_PIX_FMT_MJPEG);
+	int mjpg_index = v4l2core_get_frame_format_index(V4L2_PIX_FMT_MJPEG);
 	if(mjpg_index < 0) /*MJPG must be available for muxed uvc H264 streams*/
 		return;
 
@@ -630,7 +528,7 @@ int request_h264_frame_type(v4l2_dev_t *vd, uint16_t type)
 
 	int err = E_OK;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_PICTURE_TYPE_CONTROL, UVC_SET_CUR, &picture_type_req)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_PICTURE_TYPE_CONTROL, UVC_SET_CUR, &picture_type_req)) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_PICTURE_TYPE_CONTROL) SET_CUR error: %s\n", strerror(errno));
 	}
@@ -648,7 +546,7 @@ int request_h264_frame_type(v4l2_dev_t *vd, uint16_t type)
  *
  * returns: 0 on success or error code on fail
  */
-int v4l2core_reset_h264_encoder(v4l2_dev_t *vd)
+int h264_reset_encoder(v4l2_dev_t *vd)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -668,7 +566,7 @@ int v4l2core_reset_h264_encoder(v4l2_dev_t *vd)
  *
  * returns: video rate control mode (FIXME: 0xff on error)
  */
-uint8_t v4l2core_get_h264_video_rate_control_mode(v4l2_dev_t *vd, uint8_t query)
+uint8_t  h264_get_video_rate_control_mode(v4l2_dev_t *vd, uint8_t query)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -683,7 +581,7 @@ uint8_t v4l2core_get_h264_video_rate_control_mode(v4l2_dev_t *vd, uint8_t query)
 	uvcx_rate_control_mode_t rate_control_mode_req;
 	rate_control_mode_req.wLayerID = 0;
 
-	if((v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_RATE_CONTROL_MODE, query, &rate_control_mode_req)) < 0)
+	if((v4l2core_query_xu_control(vd->h264_unit_id, UVCX_RATE_CONTROL_MODE, query, &rate_control_mode_req)) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_RATE_CONTROL_MODE) query (%u) error: %s\n", query, strerror(errno));
 		return 0xff;
@@ -703,7 +601,7 @@ uint8_t v4l2core_get_h264_video_rate_control_mode(v4l2_dev_t *vd, uint8_t query)
  *
  * returns: error code ( 0 -OK)
  */
-int v4l2core_set_h264_video_rate_control_mode(v4l2_dev_t *vd, uint8_t mode)
+int h264_set_video_rate_control_mode(v4l2_dev_t *vd, uint8_t mode)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -721,7 +619,7 @@ int v4l2core_set_h264_video_rate_control_mode(v4l2_dev_t *vd, uint8_t mode)
 
 	int err = E_OK;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_RATE_CONTROL_MODE, UVC_SET_CUR, &rate_control_mode_req)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_RATE_CONTROL_MODE, UVC_SET_CUR, &rate_control_mode_req)) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_RATE_CONTROL_MODE) SET_CUR error: %s\n", strerror(errno));
 	}
@@ -740,7 +638,7 @@ int v4l2core_set_h264_video_rate_control_mode(v4l2_dev_t *vd, uint8_t mode)
  *
  * returns: temporal scale mode (FIXME: 0xff on error)
  */
-uint8_t v4l2core_get_h264_temporal_scale_mode(v4l2_dev_t *vd, uint8_t query)
+uint8_t h264_get_temporal_scale_mode(v4l2_dev_t *vd, uint8_t query)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -755,7 +653,7 @@ uint8_t v4l2core_get_h264_temporal_scale_mode(v4l2_dev_t *vd, uint8_t query)
 	uvcx_temporal_scale_mode_t temporal_scale_mode_req;
 	temporal_scale_mode_req.wLayerID = 0;
 
-	if(v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_TEMPORAL_SCALE_MODE, query, &temporal_scale_mode_req) < 0)
+	if(v4l2core_query_xu_control(vd->h264_unit_id, UVCX_TEMPORAL_SCALE_MODE, query, &temporal_scale_mode_req) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_TEMPORAL_SCALE_MODE) query (%u) error: %s\n", query, strerror(errno));
 		return 0xff;
@@ -775,7 +673,7 @@ uint8_t v4l2core_get_h264_temporal_scale_mode(v4l2_dev_t *vd, uint8_t query)
  *
  * returns: error code ( 0 -OK)
  */
-int v4l2core_set_h264_temporal_scale_mode(v4l2_dev_t *vd, uint8_t mode)
+int h264_set_temporal_scale_mode(v4l2_dev_t *vd, uint8_t mode)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -793,7 +691,7 @@ int v4l2core_set_h264_temporal_scale_mode(v4l2_dev_t *vd, uint8_t mode)
 
 	int err = 0;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_TEMPORAL_SCALE_MODE, UVC_SET_CUR, &temporal_scale_mode_req)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_TEMPORAL_SCALE_MODE, UVC_SET_CUR, &temporal_scale_mode_req)) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_TEMPORAL_SCALE_MODE) SET_CUR error: %s\n", strerror(errno));
 	}
@@ -812,7 +710,7 @@ int v4l2core_set_h264_temporal_scale_mode(v4l2_dev_t *vd, uint8_t mode)
  *
  * returns: temporal scale mode (FIXME: 0xff on error)
  */
-uint8_t v4l2core_get_h264_spatial_scale_mode(v4l2_dev_t *vd, uint8_t query)
+uint8_t h264_get_spatial_scale_mode(v4l2_dev_t *vd, uint8_t query)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -827,7 +725,7 @@ uint8_t v4l2core_get_h264_spatial_scale_mode(v4l2_dev_t *vd, uint8_t query)
 	uvcx_spatial_scale_mode_t spatial_scale_mode_req;
 	spatial_scale_mode_req.wLayerID = 0;
 
-	if(v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_SPATIAL_SCALE_MODE, query, &spatial_scale_mode_req) < 0)
+	if(v4l2core_query_xu_control(vd->h264_unit_id, UVCX_SPATIAL_SCALE_MODE, query, &spatial_scale_mode_req) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_SPATIAL_SCALE_MODE) query (%u) error: %s\n", query, strerror(errno));
 		return 0xff;
@@ -847,7 +745,7 @@ uint8_t v4l2core_get_h264_spatial_scale_mode(v4l2_dev_t *vd, uint8_t query)
  *
  * returns: error code ( 0 -OK)
  */
-int v4l2core_set_h264_spatial_scale_mode(v4l2_dev_t *vd, uint8_t mode)
+int h264_set_spatial_scale_mode(v4l2_dev_t *vd, uint8_t mode)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -865,7 +763,7 @@ int v4l2core_set_h264_spatial_scale_mode(v4l2_dev_t *vd, uint8_t mode)
 
 	int err = 0;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_SPATIAL_SCALE_MODE, UVC_SET_CUR, &spatial_scale_mode_req)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_SPATIAL_SCALE_MODE, UVC_SET_CUR, &spatial_scale_mode_req)) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_SPATIAL_SCALE_MODE) SET_CUR error: %s\n", strerror(errno));
 	}
@@ -884,7 +782,7 @@ int v4l2core_set_h264_spatial_scale_mode(v4l2_dev_t *vd, uint8_t mode)
  *
  * returns: frame rate config (FIXME: 0xffffffff on error)
  */
-uint32_t v4l2core_query_h264_frame_rate_config(v4l2_dev_t *vd, uint8_t query)
+uint32_t h264_query_frame_rate_config(v4l2_dev_t *vd, uint8_t query)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -899,7 +797,7 @@ uint32_t v4l2core_query_h264_frame_rate_config(v4l2_dev_t *vd, uint8_t query)
 	uvcx_framerate_config_t framerate_req;
 	framerate_req.wLayerID = 0;
 
-	if(v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_FRAMERATE_CONFIG, query, &framerate_req) < 0)
+	if(v4l2core_query_xu_control(vd->h264_unit_id, UVCX_FRAMERATE_CONFIG, query, &framerate_req) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_FRAMERATE_CONFIG) query (%u) error: %s\n", query, strerror(errno));
 		return 0xffffffff;
@@ -918,9 +816,9 @@ uint32_t v4l2core_query_h264_frame_rate_config(v4l2_dev_t *vd, uint8_t query)
  *
  * returns: frame rate config (FIXME: 0xffffffff on error)
  */
-uint32_t v4l2core_get_h264_frame_rate_config(v4l2_dev_t *vd)
+uint32_t h264_get_frame_rate_config(v4l2_dev_t *vd)
 {
-	return v4l2core_query_h264_frame_rate_config(vd, UVC_GET_CUR);
+	return h264_query_frame_rate_config(vd, UVC_GET_CUR);
 }
 
 /*
@@ -934,7 +832,7 @@ uint32_t v4l2core_get_h264_frame_rate_config(v4l2_dev_t *vd)
  *
  * returns: error code ( 0 -OK)
  */
-int v4l2core_set_h264_frame_rate_config(v4l2_dev_t *vd, uint32_t framerate)
+int h264_set_frame_rate_config(v4l2_dev_t *vd, uint32_t framerate)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -952,7 +850,7 @@ int v4l2core_set_h264_frame_rate_config(v4l2_dev_t *vd, uint32_t framerate)
 
 	int err = 0;
 
-	if((err = v4l2core_query_xu_control(vd, vd->h264_unit_id, UVCX_FRAMERATE_CONFIG, UVC_SET_CUR, &framerate_req)) < 0)
+	if((err = v4l2core_query_xu_control(vd->h264_unit_id, UVCX_FRAMERATE_CONFIG, UVC_SET_CUR, &framerate_req)) < 0)
 	{
 		fprintf(stderr, "V4L2_CORE: (UVCX_FRAMERATE_CONFIG) SET_CUR error: %s\n", strerror(errno));
 	}
@@ -973,7 +871,7 @@ int v4l2core_set_h264_frame_rate_config(v4l2_dev_t *vd, uint32_t framerate)
  *
  * returns: error code ( 0 -OK)
  */
-int v4l2core_probe_h264_config_probe_req(
+int h264_probe_config_probe_req(
 			v4l2_dev_t *vd,
 			uint8_t query,
 			uvcx_video_config_probe_commit_t *config_probe_req)
