@@ -34,6 +34,7 @@
 
 #include "gviewv4l2core.h"
 #include "v4l2_controls.h"
+#include "v4l2_xu_ctrls.h"
 #include "../config.h"
 
 #ifndef V4L2_CTRL_ID2CLASS
@@ -42,6 +43,9 @@
 
 extern int verbosity;
 
+// GUID for logitech peripheral (pan/tilt) V3 extension unit: {FFE52D21-8030-4E2C-82d9-f587d00540bd}
+#define GUID_LOGITECH_PERIPHERAL_XU {0x21, 0x2D, 0xE5, 0xFF, 0x30, 0x80, 0x2C, 0x4E, 0x82, 0xD9, 0xF5, 0x87, 0xD0, 0x05, 0x40, 0xBD}
+
 /*
  * needed only for language files (not used)
  */
@@ -153,14 +157,17 @@ static int query_ioctl(v4l2_dev_t *vd, int current_ctrl, struct v4l2_queryctrl*
  *   i - control index (from control list)
  *
  * asserts:
- *   control is not null
+ *   none
  *
  * returns: void
  */
 static void print_control(v4l2_ctrl_t *control, int i)
 {
-	/*assertions*/
-	assert(control != NULL);
+	if(control == NULL)
+	{
+		printf("V4L2_CORE: null control at index %i\n", i);
+		return;
+	}
 
 	int j=0;
 
@@ -237,7 +244,6 @@ static void print_control(v4l2_ctrl_t *control, int i)
  *
  * asserts:
  *   vd is not null
- *   vd->list_device_controls is not null
  *
  * returns: void
  */
@@ -245,7 +251,12 @@ static void print_control_list(v4l2_dev_t *vd)
 {
 	/*asserts*/
 	assert(vd != NULL);
-	assert(vd->list_device_controls != NULL);
+	
+	if(vd->list_device_controls == NULL)
+	{
+		printf("V4L2_CORE: WARNING empty control list\n");
+		return;
+	}
 
 	int i = 0;
 	v4l2_ctrl_t *current = vd->list_device_controls;
@@ -257,6 +268,28 @@ static void print_control_list(v4l2_dev_t *vd)
     }
 }
 
+
+/*
+ * gets the logitech peripheral (pan/tilt) V3 xu control unit id, if any
+ * args:
+ *   vd - pointer to video device data
+ *
+ * asserts:
+ *   vd is not null
+ *   device_list->list_devices is not null
+ *
+ * returns: unit id or 0 if none
+ */
+static uint8_t get_logitech_peripheral_unit_id (v4l2_dev_t *vd)
+{
+	if(verbosity > 1)
+		printf("V4L2_CORE: checking for logitech peripheral unit id\n");
+	
+	uint8_t guid[16] =  GUID_LOGITECH_PERIPHERAL_XU;	
+	return get_guid_unit_id (vd, guid);
+}
+
+
 /*
  * add control to control list
  * args:
@@ -363,8 +396,11 @@ static v4l2_ctrl_t *add_control(v4l2_dev_t *vd, struct v4l2_queryctrl* queryctrl
 	/*check for pan/tilt control*/
 	else if(queryctrl->id == V4L2_CID_TILT_RELATIVE ||
 			queryctrl->id == V4L2_CID_PAN_RELATIVE)
+	{
+		/*get unit id for logitech pan_tilt V3 if any*/
 		vd->has_pantilt_control_id = 1;
-
+		vd->pantilt_unit_id = get_logitech_peripheral_unit_id(vd);
+	}
     // Add the control to the linked list
     control = calloc (1, sizeof(v4l2_ctrl_t));
     if(control == NULL)
@@ -538,7 +574,7 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
     {
         case V4L2_CID_EXPOSURE_AUTO:
             {
-                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id(vd, id);
+                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id(id);
                 if(ctrl_this == NULL)
                     break;
 
@@ -546,16 +582,16 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
                 {
                     case V4L2_EXPOSURE_AUTO:
                         {
-                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
 
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_RELATIVE );
                             if (ctrl_that)
                                 ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_EXPOSURE_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
@@ -564,15 +600,15 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
 
                     case V4L2_EXPOSURE_APERTURE_PRIORITY:
                         {
-                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_EXPOSURE_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_RELATIVE );
                             if (ctrl_that)
                                 ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
@@ -581,16 +617,16 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
 
                     case V4L2_EXPOSURE_SHUTTER_PRIORITY:
                         {
-                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
 
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_RELATIVE );
                             if (ctrl_that)
                                 ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_EXPOSURE_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
@@ -599,15 +635,15 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
 
                     default:
                         {
-                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                            v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_EXPOSURE_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_ABSOLUTE );
                             if (ctrl_that)
                                 ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
-                            ctrl_that = v4l2core_get_control_by_id(vd,
+                            ctrl_that = v4l2core_get_control_by_id(
                                 V4L2_CID_IRIS_RELATIVE );
                             if (ctrl_that)
                                 ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
@@ -619,29 +655,29 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
 
         case V4L2_CID_FOCUS_AUTO:
             {
-                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id(vd, id );
+                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id( id );
                 if(ctrl_this == NULL)
                     break;
                 if(ctrl_this->value > 0)
                 {
-                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_FOCUS_ABSOLUTE);
                     if (ctrl_that)
                         ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
 
-                    ctrl_that = v4l2core_get_control_by_id(vd,
+                    ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_FOCUS_RELATIVE);
                     if (ctrl_that)
                         ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
                 }
                 else
                 {
-                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_FOCUS_ABSOLUTE);
                     if (ctrl_that)
                         ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
 
-                    ctrl_that = v4l2core_get_control_by_id(vd,
+                    ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_FOCUS_RELATIVE);
                     if (ctrl_that)
                         ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
@@ -651,19 +687,19 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
 
         case V4L2_CID_HUE_AUTO:
             {
-                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id(vd, id );
+                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id( id );
                 if(ctrl_this == NULL)
                     break;
                 if(ctrl_this->value > 0)
                 {
-                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_HUE);
                     if (ctrl_that)
                         ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
                 }
                 else
                 {
-                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_HUE);
                     if (ctrl_that)
                         ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
@@ -673,36 +709,36 @@ static void update_ctrl_flags(v4l2_dev_t *vd, int id)
 
         case V4L2_CID_AUTO_WHITE_BALANCE:
             {
-                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id(vd, id );
+                v4l2_ctrl_t *ctrl_this = v4l2core_get_control_by_id( id );
                 if(ctrl_this == NULL)
                     break;
 
                 if(ctrl_this->value > 0)
                 {
-                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_WHITE_BALANCE_TEMPERATURE);
                     if (ctrl_that)
                         ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
-                    ctrl_that = v4l2core_get_control_by_id(vd,
+                    ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_BLUE_BALANCE);
                     if (ctrl_that)
                         ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
-                    ctrl_that = v4l2core_get_control_by_id(vd,
+                    ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_RED_BALANCE);
                     if (ctrl_that)
                         ctrl_that->control.flags |= V4L2_CTRL_FLAG_GRABBED;
                 }
                 else
                 {
-                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(vd,
+                    v4l2_ctrl_t *ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_WHITE_BALANCE_TEMPERATURE);
                     if (ctrl_that)
                         ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
-                    ctrl_that = v4l2core_get_control_by_id(vd,
+                    ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_BLUE_BALANCE);
                     if (ctrl_that)
                         ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
-                    ctrl_that = v4l2core_get_control_by_id(vd,
+                    ctrl_that = v4l2core_get_control_by_id(
                         V4L2_CID_RED_BALANCE);
                     if (ctrl_that)
                         ctrl_that->control.flags &= !(V4L2_CTRL_FLAG_GRABBED);
@@ -752,41 +788,14 @@ void disable_special_auto (v4l2_dev_t *vd, int id)
 	/*asserts*/
 	assert(vd != NULL);
 
-    v4l2_ctrl_t *current = v4l2core_get_control_by_id(vd, id);
+    v4l2_ctrl_t *current = v4l2core_get_control_by_id( id);
     if(current && ((id == V4L2_CID_FOCUS_AUTO) || (id == V4L2_CID_HUE_AUTO)))
     {
         current->value = 0;
-        v4l2core_set_control_value_by_id(vd, id);
+        v4l2core_set_control_value_by_id( id);
     }
 }
 
-/*
- * return the control associated to id from device list
- * args:
- *   vd - pointer to video device data
- *
- * asserts:
- *   vd is not null
- *   vd->list_device_controls is not null
- *
- * returns: pointer to v4l2_control if succeded or null otherwise
- */
-v4l2_ctrl_t *v4l2core_get_control_by_id(v4l2_dev_t *vd, int id)
-{
-	/*asserts*/
-	assert(vd != NULL);
-	assert(vd->list_device_controls != NULL);
-
-    v4l2_ctrl_t *current = vd->list_device_controls;
-    for(; current != NULL; current = current->next)
-    {
-        if(current->control.id == id)
-            return (current);
-    }
-
-    return(NULL);
-}
-
 /*
  * goes trough the control list and updates/retrieves current values
  * args:
@@ -795,7 +804,6 @@ v4l2_ctrl_t *v4l2core_get_control_by_id(v4l2_dev_t *vd, int id)
  * asserts:
  *   vd is not null
  *   vd->fd is valid
- *   vd->list_device_controls is not null
  *
  * returns: void
  */
@@ -804,7 +812,12 @@ void get_v4l2_control_values (v4l2_dev_t *vd)
     /*asserts*/
     assert(vd != NULL);
     assert(vd->fd > 0);
-    assert(vd->list_device_controls != NULL);
+
+    if(vd->list_device_controls == NULL)
+	{
+		printf("V4L2_CORE: (get control values) empty control list\n");
+		return;
+	}
 
     int ret = 0;
     struct v4l2_ext_control clist[vd->num_controls];
@@ -885,7 +898,7 @@ void get_v4l2_control_values (v4l2_dev_t *vd)
             //fill in the values on the control list
             for(i=0; i<count; i++)
             {
-                v4l2_ctrl_t *ctrl = v4l2core_get_control_by_id(vd, clist[i].id);
+                v4l2_ctrl_t *ctrl = v4l2core_get_control_by_id(clist[i].id);
                 if(!ctrl)
                 {
                     fprintf(stderr, "V4L2_CORE: couldn't get control for id: %i\n", clist[i].id);
@@ -935,11 +948,41 @@ void get_v4l2_control_values (v4l2_dev_t *vd)
     update_ctrl_list_flags(vd);
 }
 
+/*
+ * return the control associated to id from device list
+ * args:
+ *   vd - pointer to video device data
+ *   id - control id
+ *
+ * asserts:
+ *   vd is not null
+ *
+ * returns: pointer to v4l2_control if succeded or null otherwise
+ */
+v4l2_ctrl_t *get_control_by_id(v4l2_dev_t *vd, int id)
+{
+	/*asserts*/
+	assert(vd != NULL);
+	
+	v4l2_ctrl_t *current = vd->list_device_controls;
+    for(; current != NULL; current = current->next)
+    {
+		if(current == NULL)
+			break;
+
+        if(current->control.id == id)
+            return (current);
+    }
+
+    return(NULL);
+}
+
 /*
  * updates the value for control id from the device
  * also updates control flags
  * args:
  *   vd - pointer to video device data
+ *   id - control id
  *
  * asserts:
  *   vd is not null
@@ -947,13 +990,13 @@ void get_v4l2_control_values (v4l2_dev_t *vd)
  *
  * returns: ioctl result
  */
-int v4l2core_get_control_value_by_id (v4l2_dev_t *vd, int id)
+int get_control_value_by_id (v4l2_dev_t *vd, int id)
 {
 	/*asserts*/
 	assert(vd != NULL);
 	assert(vd->fd > 0);
 
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(vd, id );
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id );
     int ret = 0;
 
     if(!control)
@@ -1060,7 +1103,6 @@ int v4l2core_get_control_value_by_id (v4l2_dev_t *vd, int id)
  * asserts:
  *   vd is not null
  *   vd->fd is valid
- *   vd->list_device_controls is not null
  *
  * returns: void
  */
@@ -1069,7 +1111,12 @@ void set_v4l2_control_values (v4l2_dev_t *vd)
 	/*asserts*/
 	assert(vd != NULL);
 	assert(vd->fd > 0);
-	assert(vd->list_device_controls != NULL);
+	
+	if(vd->list_device_controls == NULL)
+	{
+		printf("V4L2_CORE: (set control values) empty control list\n");
+		return;
+	}
 
     int ret = 0;
     struct v4l2_ext_control clist[vd->num_controls];
@@ -1157,7 +1204,7 @@ void set_v4l2_control_values (v4l2_dev_t *vd)
                         ret = xioctl(vd->fd, VIDIOC_S_CTRL, &ctrl);
                         if(ret)
                         {
-                            v4l2_ctrl_t *ctrl = v4l2core_get_control_by_id(vd, clist[i].id);
+                            v4l2_ctrl_t *ctrl = v4l2core_get_control_by_id(clist[i].id);
                             if(ctrl)
                                 fprintf(stderr, "V4L2_CORE: control(0x%08x) \"%s\" failed to set (error %i)\n",
                                     clist[i].id, ctrl->control.name, ret);
@@ -1177,7 +1224,7 @@ void set_v4l2_control_values (v4l2_dev_t *vd)
                         ctrls.controls = &clist[i];
                         ret = xioctl(vd->fd, VIDIOC_S_EXT_CTRLS, &ctrls);
 
-                        v4l2_ctrl_t *ctrl = v4l2core_get_control_by_id(vd, clist[i].id);
+                        v4l2_ctrl_t *ctrl = v4l2core_get_control_by_id(clist[i].id);
 
                         if(ret)
                         {
@@ -1211,15 +1258,19 @@ void set_v4l2_control_values (v4l2_dev_t *vd)
  *
  * asserts:
  *   vd is not null
- *   vd->list_device_controls is not null
  *
  * returns: void
  */
-void v4l2core_set_control_defaults(v4l2_dev_t *vd)
+void set_control_defaults(v4l2_dev_t *vd)
 {
 	/*asserts*/
 	assert(vd != NULL);
-	assert(vd->list_device_controls != NULL);
+	
+	if(vd->list_device_controls == NULL)
+	{
+		printf("V4L2_CORE: (set control defaults) empty control list\n");
+		return;
+	}
 
     v4l2_ctrl_t *current = vd->list_device_controls;
     v4l2_ctrl_t *next = current->next;
@@ -1263,6 +1314,7 @@ void v4l2core_set_control_defaults(v4l2_dev_t *vd)
  * sets the value of control id in device
  * args:
  *   vd - pointer to video device data
+ *   id - control id
  *
  * asserts:
  *   vd is not null
@@ -1270,19 +1322,34 @@ void v4l2core_set_control_defaults(v4l2_dev_t *vd)
  *
  * returns: ioctl result
  */
-int v4l2core_set_control_value_by_id(v4l2_dev_t *vd, int id)
+int set_control_value_by_id(v4l2_dev_t *vd, int id)
 {
 	/*asserts*/
 	assert(vd != NULL);
 	assert(vd->fd > 0);
 
-    v4l2_ctrl_t *control = v4l2core_get_control_by_id(vd, id );
+    v4l2_ctrl_t *control = v4l2core_get_control_by_id(id);
     int ret = 0;
 
     if(!control)
         return (-1);
     if(control->control.flags & V4L2_CTRL_FLAG_READ_ONLY)
         return (-1);
+        
+    if((id == V4L2_CID_PAN_RELATIVE || id == V4L2_CID_TILT_RELATIVE) &&
+		vd->pantilt_unit_id > 0)
+	{
+		/*use raw control in this case - prevents uvcvideo cache bug*/
+		uint32_t pantilt = 0;
+		if(id == V4L2_CID_PAN_RELATIVE)
+			pantilt |= (int16_t) control->value;
+		else
+			pantilt |= ((int16_t) control->value) << 16;
+			
+		return query_xu_control(vd, vd->pantilt_unit_id, 1, UVC_SET_CUR, &pantilt);
+	}
+
+		
 
     if( control->class == V4L2_CTRL_CLASS_USER
 #ifdef V4L2_CTRL_TYPE_STRING
@@ -1358,7 +1425,7 @@ int v4l2core_set_control_value_by_id(v4l2_dev_t *vd, int id)
     }
 
     //update real value
-    v4l2core_get_control_value_by_id(vd, id);
+    get_control_value_by_id(vd, id);
 
     return (ret);
 }
@@ -1370,7 +1437,6 @@ int v4l2core_set_control_value_by_id(v4l2_dev_t *vd, int id)
  *
  * asserts:
  *   vd is not null
- *   vd->list_device_controls is not null
  *
  * returns: void
  */
@@ -1378,7 +1444,11 @@ void free_v4l2_control_list(v4l2_dev_t *vd)
 {
 	/*asserts*/
 	assert(vd != NULL);
-	assert(vd->list_device_controls != NULL);
+	
+	if(vd->list_device_controls == NULL)
+	{
+		return;
+	}
 
 	v4l2_ctrl_t *first = vd->list_device_controls;
 
