@@ -437,29 +437,72 @@ static int enum_frame_sizes(v4l2_dev_t *vd, uint32_t pixfmt, int fmtind)
 			if (ret != 0)
 				fprintf(stderr, "V4L2_CORE:  Unable to enumerate frame sizes %s\n", strerror(ret));
 		}
-		else if (fsize.type == V4L2_FRMSIZE_TYPE_CONTINUOUS)
+		else if (fsize.type == V4L2_FRMSIZE_TYPE_CONTINUOUS || fsize.type == V4L2_FRMSIZE_TYPE_STEPWISE)
 		{
 			if(verbosity > 0)
 			{
-				printf("{ continuous: min { width = %u, height = %u } .. "
-					"max { width = %u, height = %u } }\n",
-					fsize.stepwise.min_width, fsize.stepwise.min_height,
-					fsize.stepwise.max_width, fsize.stepwise.max_height);
-				printf("  will not enumerate frame intervals.\n");
-			}
-		}
-		else if (fsize.type == V4L2_FRMSIZE_TYPE_STEPWISE)
-		{
-			if(verbosity > 0)
-			{
-				printf("{ stepwise: min { width = %u, height = %u } .. "
-					"max { width = %u, height = %u } / "
-					"stepsize { width = %u, height = %u } }\n",
-					fsize.stepwise.min_width, fsize.stepwise.min_height,
-					fsize.stepwise.max_width, fsize.stepwise.max_height,
-					fsize.stepwise.step_width, fsize.stepwise.step_height);
-				printf("  will not enumerate frame intervals.\n");
+				if(fsize.type == V4L2_FRMSIZE_TYPE_CONTINUOUS)
+					printf("{ continuous: min { width = %u, height = %u } .. "
+						"max { width = %u, height = %u } }\n",
+						fsize.stepwise.min_width, fsize.stepwise.min_height,
+						fsize.stepwise.max_width, fsize.stepwise.max_height);
+				else
+					printf("{ stepwise: min { width = %u, height = %u } .. "
+						"max { width = %u, height = %u } / "
+						"stepsize { width = %u, height = %u } }\n",
+						fsize.stepwise.min_width, fsize.stepwise.min_height,
+						fsize.stepwise.max_width, fsize.stepwise.max_height,
+						fsize.stepwise.step_width, fsize.stepwise.step_height);
 			}
+				
+			/*add at least min and max values*/
+			fsizeind++; /*min*/
+				
+			vd->list_stream_formats[fmtind-1].list_stream_cap = realloc(
+				vd->list_stream_formats[fmtind-1].list_stream_cap,
+				fsizeind * sizeof(v4l2_stream_cap_t));
+
+			assert(vd->list_stream_formats[fmtind-1].list_stream_cap != NULL);
+				
+			vd->list_stream_formats[fmtind-1].numb_res = fsizeind;
+
+			vd->list_stream_formats[fmtind-1].list_stream_cap[fsizeind-1].width = fsize.stepwise.min_width;
+			vd->list_stream_formats[fmtind-1].list_stream_cap[fsizeind-1].height = fsize.stepwise.min_height;
+
+			ret = enum_frame_intervals(vd,
+				pixfmt,
+				fsize.stepwise.min_width,
+				fsize.stepwise.min_height,
+				fmtind,
+				fsizeind);
+
+			if (ret != 0)
+				fprintf(stderr, "V4L2_CORE:  Unable to enumerate frame sizes %s\n", strerror(ret));
+					
+				
+			fsizeind++; /*max*/
+				
+			vd->list_stream_formats[fmtind-1].list_stream_cap = realloc(
+				vd->list_stream_formats[fmtind-1].list_stream_cap,
+				fsizeind * sizeof(v4l2_stream_cap_t));
+
+			assert(vd->list_stream_formats[fmtind-1].list_stream_cap != NULL);
+				
+			vd->list_stream_formats[fmtind-1].numb_res = fsizeind;
+
+			vd->list_stream_formats[fmtind-1].list_stream_cap[fsizeind-1].width = fsize.stepwise.max_width;
+			vd->list_stream_formats[fmtind-1].list_stream_cap[fsizeind-1].height = fsize.stepwise.max_height;
+
+			ret = enum_frame_intervals(vd,
+				pixfmt,
+				fsize.stepwise.max_width,
+				fsize.stepwise.max_height,
+				fmtind,
+				fsizeind);
+
+			if (ret != 0)
+				fprintf(stderr, "V4L2_CORE:  Unable to enumerate frame sizes %s\n", strerror(ret));	
+				
 		}
 		else
 		{
@@ -498,6 +541,14 @@ static int enum_frame_sizes(v4l2_dev_t *vd, uint32_t pixfmt, int fmtind)
 		int width = fmt.fmt.pix.width;
 		int height = fmt.fmt.pix.height;
 
+		if(width <= 0 || height <= 0)
+		{
+			printf("{ VIDIOC_TRY_FMT (invalid values): width = %u, height = %u }\n",
+				vd->format.fmt.pix.width,
+				vd->format.fmt.pix.height);
+			return EINVAL;
+		}
+		
 		if(verbosity > 0)
 		{
 			printf("{ VIDIOC_TRY_FMT : width = %u, height = %u }\n",
@@ -556,7 +607,7 @@ static int enum_frame_sizes(v4l2_dev_t *vd, uint32_t pixfmt, int fmtind)
  *   vd->fd is valid ( > 0 )
  *   vd->list_stream_formats is null
  *
- * returns: 0 if enumeration succeded or errno otherwise
+ * returns: 0 (E_OK) if enumeration succeded or error otherwise
  */
 int enum_frame_formats(v4l2_dev_t *vd)
 {
@@ -568,6 +619,7 @@ int enum_frame_formats(v4l2_dev_t *vd)
 	int ret=E_OK;
 
 	int fmtind=0;
+	int valid_formats=0; /*number of valid formats found (with valid frame sizes)*/
 	struct v4l2_fmtdesc fmt;
 	memset(&fmt, 0, sizeof(fmt));
 	fmt.index = 0;
@@ -616,13 +668,18 @@ int enum_frame_formats(v4l2_dev_t *vd)
 		ret = enum_frame_sizes(vd, fmt.pixelformat, fmtind);
 		if (ret != 0)
 			fprintf( stderr, "v4L2_CORE: Unable to enumerate frame sizes :%s\n", strerror(ret));
+		
+		if(dec_support && !ret)
+			valid_formats++; /*the format can be decoded and it has valid frame sizes*/
 	}
 
 	if (errno != EINVAL)
 		fprintf( stderr, "v4L2_CORE: (VIDIOC_ENUM_FMT) - Error enumerating frame formats: %s\n", strerror(errno));
 
-
-	return (ret);
+	if(valid_formats > 0)
+		return E_OK;
+	else
+		return E_DEVICE_ERR;
 }
 
 /* get frame format index from format list
@@ -636,7 +693,7 @@ int enum_frame_formats(v4l2_dev_t *vd)
  *
  * returns: format list index or -1 if not available
  */
-int v4l2core_get_frame_format_index(v4l2_dev_t *vd, int format)
+int get_frame_format_index(v4l2_dev_t *vd, int format)
 {
 	/*asserts*/
 	assert(vd != NULL);
@@ -666,7 +723,7 @@ int v4l2core_get_frame_format_index(v4l2_dev_t *vd, int format)
  *
  * returns: resolution list index for format index or -1 if not available
  */
-int v4l2core_get_format_resolution_index(v4l2_dev_t *vd, int format, int width, int height)
+int get_format_resolution_index(v4l2_dev_t *vd, int format, int width, int height)
 {
 	/*asserts*/
 	assert(vd != NULL);
